
module Wordle where

import System.Random (newStdGen)
import System.Random.Shuffle

change :: [Char] -> Int -> [Char]
change word nb = take (nb-1) word ++ "-" ++ drop nb word
dropElem :: Eq t => [t] -> t -> [t]

dropElem [] _ = []
dropElem (x:xl) nb
    | x == nb = xl
    | otherwise = x : dropElem xl nb

testResult :: String -> String -> String
testResult xl sol = testPlace sol xl (testResult' xl sol)
    where
        testResult' [] _ = []
        testResult' _ [] = []
        testResult' (x:xl) (s:sl)
            | x == s    = True : testResult' xl sl
            | otherwise = False : testResult' xl sl
        testPlace sol sl wl = testPlace' (map snd $ filter (not . fst) $ zip wl sol) sl wl
            where
                testPlace' _ [] _ = []
                testPlace' _ (_:_) [] = []
                testPlace' xl (s:sl) (w:wl)
                    | w                 = 'O' : testPlace' xl sl wl
                    | s `elem` xl       = '-' : testPlace' (dropElem xl s) sl wl
                    | otherwise         = 'X' : testPlace' xl sl wl

testWin :: String -> Bool
testWin = all (== 'O')


game :: String -> IO()
game x = do
    putStrLn $ "Your turn (the word have " ++ show (length x) ++ " letters):"
    line <- getLine
    if null line || (length line /= length x)
        then game x
        else do
            let res = testResult line x
            putStrLn res
            if testWin res
                then return ()
                else game x


-- make a real error if empty list
startGame :: [String] -> IO()
startGame [] = print "Empty list"
startGame x = do
    rng <- newStdGen
    game $ head $ shuffle' x (length x) rng