
module WordleQuickCheck where

import Test.QuickCheck
import Wordle ( testResult )

letter :: Gen String
letter = listOf (elements ['a'..'z'])

letterN :: Int -> Gen String
letterN n = vectorOf n (elements ['a'..'z'])

letterDouble :: Gen (String,String)
letterDouble = do
    a <- choose (0, 255)
    (,) <$> letterN a <*> letterN a

testResultSameLength :: Property
testResultSameLength = forAll letterDouble (\(n,m) -> length (testResult m n) == length n)

testResultGoodAnswer :: Property
testResultGoodAnswer = forAll letter (\n -> do
    let a = testResult n n in
        length a == length n && a == replicate (length a) 'O'
    )

testResultGoodChar :: Property
testResultGoodChar = forAll letterDouble (\(n,m) -> do
    let a = testResult m n in
        all (`elem` "X-O") a
    )

deepCheck :: Property -> IO ()
deepCheck = quickCheckWith (stdArgs {maxSuccess = 10000})

makeAllTest :: IO ()
makeAllTest = do
    deepCheck testResultSameLength
    deepCheck testResultGoodChar
    deepCheck testResultGoodAnswer